@@ -1,10 +1,11 @@
 use std::{cmp::max, collections::HashMap, ptr::null_mut, sync::Mutex};
 
 use crate::{
-    handles::{self, OutputStringBuffer},
+    handles::{self, log_diagnostics, OutputStringBuffer, SqlResult, State},
     Connection, Error,
 };
-use odbc_sys::{AttrOdbcVersion, FetchOrientation, HWnd};
+use log::debug;
+use odbc_sys::{AttrCpMatch, AttrOdbcVersion, FetchOrientation, HWnd};
 use widestring::{U16CStr, U16Str, U16String};
 
 // Currently only windows driver manager supports prompt.
@@ -82,7 +83,10 @@ impl Environment {
     pub unsafe fn set_connection_pooling(
         scheme: odbc_sys::AttrConnectionPooling,
     ) -> Result<(), Error> {
-        handles::Environment::set_connection_pooling(scheme)
+        match handles::Environment::set_connection_pooling(scheme) {
+            SqlResult::Error { .. } => Err(Error::NoDiagnostics),
+            SqlResult::Success(()) | SqlResult::SuccessWithInfo(()) => Ok(()),
+        }
     }
 
     /// Determines how a connection is chosen from a connection pool. When [`Self::connect`],
@@ -99,11 +103,11 @@ impl Environment {
     /// the default.
     /// * [`crate::sys::AttrCpMatch::Relaxed`] = Connections with matching connection string \
     /// keywords can be used. Keywords must match, but not all connection attributes must match.
-    pub fn set_connection_pooling_matching(
-        &mut self,
-        matching: odbc_sys::AttrCpMatch,
-    ) -> Result<(), Error> {
-        self.environment.set_connection_pooling_matching(matching)
+    pub fn set_connection_pooling_matching(&mut self, matching: AttrCpMatch) -> Result<(), Error> {
+        match self.environment.set_connection_pooling_matching(matching) {
+            SqlResult::Error { .. } => Err(Error::NoDiagnostics),
+            SqlResult::Success(()) | SqlResult::SuccessWithInfo(()) => Ok(()),
+        }
     }
 
     /// Entry point into this API. Allocates a new ODBC Environment and declares to the driver
@@ -119,8 +123,37 @@ impl Environment {
     ///
     /// Creating one environment in your binary is safe however.
     pub unsafe fn new() -> Result<Self, Error> {
-        let environment = crate::handles::Environment::new()?;
-        environment.declare_version(AttrOdbcVersion::Odbc3_80)?;
+        let result = handles::Environment::new();
+
+        let environment = match result {
+            SqlResult::Success(env) => env,
+            SqlResult::SuccessWithInfo(env) => {
+                log_diagnostics(&env);
+                env
+            }
+            SqlResult::Error { .. } => return Err(Error::NoDiagnostics),
+        };
+
+        debug!("ODBC Environment created.");
+
+        let result = environment
+            .declare_version(AttrOdbcVersion::Odbc3_80)
+            .into_result(&environment);
+
+        // Translate invalid attribute into a more meaningful error, provided the additional
+        // context that we know we tried to set version number.
+        result.map_err(|error| {
+            if let Error::Diagnostics { record, function } = error {
+                if record.state == State::INVALID_STATE_TRANSACTION {
+                    Error::UnsupportedOdbcApiVersion(record)
+                } else {
+                    Error::Diagnostics { record, function }
+                }
+            } else {
+                error
+            }
+        })?;
+
         Ok(Self {
             environment,
             info_iterator_state: Mutex::new(()),
@@ -188,8 +221,13 @@ impl Environment {
         user: &U16Str,
         pwd: &U16Str,
     ) -> Result<Connection<'_>, Error> {
-        let mut connection = self.environment.allocate_connection()?;
-        connection.connect(data_source_name, user, pwd)?;
+        let mut connection = self
+            .environment
+            .allocate_connection()
+            .into_result(&self.environment)?;
+        connection
+            .connect(data_source_name, user, pwd)
+            .into_result(&connection)?;
         Ok(Connection::new(connection))
     }
 
@@ -241,8 +279,13 @@ impl Environment {
         &self,
         connection_string: &U16Str,
     ) -> Result<Connection<'_>, Error> {
-        let mut connection = self.environment.allocate_connection()?;
-        connection.connect_with_connection_string(connection_string)?;
+        let mut connection = self
+            .environment
+            .allocate_connection()
+            .into_result(&self.environment)?;
+        connection
+            .connect_with_connection_string(connection_string)
+            .into_result(&connection)?;
         Ok(Connection::new(connection))
     }
 
@@ -355,7 +398,10 @@ impl Environment {
         completed_connection_string: Option<&mut OutputStringBuffer>,
         driver_completion: DriverCompleteOption<'_>,
     ) -> Result<Connection<'_>, Error> {
-        let mut connection = self.environment.allocate_connection()?;
+        let mut connection = self
+            .environment
+            .allocate_connection()
+            .into_result(&self.environment)?;
         let connection_string = U16String::from_str(connection_string);
 
         unsafe {
@@ -364,8 +410,10 @@ impl Environment {
                 driver_completion.parent_window(),
                 completed_connection_string,
                 driver_completion.as_sys(),
-            )?;
+            )
         }
+        .map(|res| res.into_result(&connection))
+        .unwrap_or(Err(Error::AbortedConnectionStringCompletion))?;
         Ok(Connection::new(connection))
     }
 
@@ -393,22 +441,21 @@ impl Environment {
         let _lock = self.info_iterator_state.lock().unwrap();
         unsafe {
             // Find required buffer size to avoid truncation.
-            let (mut desc_len, mut attr_len) = if let Some(v) = self
+            let (mut desc_len, mut attr_len) = if let Some(res) = self
                 .environment
                 // Start with first so we are independent of state
-                .drivers_buffer_len(FetchOrientation::First)?
+                .drivers_buffer_len(FetchOrientation::First)
             {
-                v
+                res.into_result(&self.environment)?
             } else {
                 // No drivers present
                 return Ok(Vec::new());
             };
 
             // If there are let's loop over the rest
-            while let Some((candidate_desc_len, candidate_attr_len)) = self
-                .environment
-                .drivers_buffer_len(FetchOrientation::Next)?
-            {
+            while let Some(res) = self.environment.drivers_buffer_len(FetchOrientation::Next) {
+                let (candidate_desc_len, candidate_attr_len) =
+                    res.into_result(&self.environment)?;
                 desc_len = max(candidate_desc_len, desc_len);
                 attr_len = max(candidate_attr_len, attr_len);
             }
@@ -417,11 +464,13 @@ impl Environment {
             let mut desc_buf = vec![0; desc_len as usize + 1];
             let mut attr_buf = vec![0; attr_len as usize + 1];
 
-            while self.environment.drivers_buffer_fill(
-                FetchOrientation::Next,
-                &mut desc_buf,
-                &mut attr_buf,
-            )? {
+            while self
+                .environment
+                .drivers_buffer_fill(FetchOrientation::Next, &mut desc_buf, &mut attr_buf)
+                .map(|res| res.into_result(&self.environment))
+                .transpose()?
+                .is_some()
+            {
                 let description = U16CStr::from_slice_with_nul(&desc_buf).unwrap();
                 let attributes = U16CStr::from_slice_with_nul(&attr_buf).unwrap();
 
@@ -503,8 +552,8 @@ impl Environment {
         unsafe {
             // Find required buffer size to avoid truncation.
             let (mut server_name_len, mut driver_len) =
-                if let Some(v) = self.environment.data_source_buffer_len(direction)? {
-                    v
+                if let Some(res) = self.environment.data_source_buffer_len(direction) {
+                    res.into_result(&self.environment)?
                 } else {
                     // No drivers present
                     return Ok(Vec::new());
@@ -513,7 +562,9 @@ impl Environment {
             // If there are let's loop over the rest
             while let Some((candidate_name_len, candidate_decs_len)) = self
                 .environment
-                .drivers_buffer_len(FetchOrientation::Next)?
+                .drivers_buffer_len(FetchOrientation::Next)
+                .map(|res| res.into_result(&self.environment))
+                .transpose()?
             {
                 server_name_len = max(candidate_name_len, server_name_len);
                 driver_len = max(candidate_decs_len, driver_len);
@@ -523,11 +574,13 @@ impl Environment {
             let mut server_name_buf = vec![0; server_name_len as usize + 1];
             let mut driver_buf = vec![0; driver_len as usize + 1];
 
-            let mut not_empty = self.environment.data_source_buffer_fill(
-                direction,
-                &mut server_name_buf,
-                &mut driver_buf,
-            )?;
+            let mut not_empty = self
+                .environment
+                .data_source_buffer_fill(direction, &mut server_name_buf, &mut driver_buf)
+                .map(|res| res.into_result(&self.environment))
+                .transpose()?
+                .is_some();
+
             while not_empty {
                 let server_name = U16CStr::from_slice_with_nul(&server_name_buf).unwrap();
                 let driver = U16CStr::from_slice_with_nul(&driver_buf).unwrap();
@@ -539,11 +592,16 @@ impl Environment {
                     server_name,
                     driver,
                 });
-                not_empty = self.environment.data_source_buffer_fill(
-                    FetchOrientation::Next,
-                    &mut server_name_buf,
-                    &mut driver_buf,
-                )?;
+                not_empty = self
+                    .environment
+                    .data_source_buffer_fill(
+                        FetchOrientation::Next,
+                        &mut server_name_buf,
+                        &mut driver_buf,
+                    )
+                    .map(|res| res.into_result(&self.environment))
+                    .transpose()?
+                    .is_some()
             }
         }
 
