@@ -1,11 +1,12 @@
 use super::{
-    as_handle::AsHandle, drop_handle, error::IntoResult, logging::log_diagnostics, Connection,
-    Error, State,
+    as_handle::AsHandle,
+    drop_handle,
+    sql_result::{ExtSqlReturn, SqlResult},
+    Connection,
 };
-use log::debug;
 use odbc_sys::{
-    AttrOdbcVersion, EnvironmentAttribute, FetchOrientation, HDbc, HEnv, Handle, HandleType,
-    SQLAllocHandle, SQLDataSourcesW, SQLDriversW, SQLSetEnvAttr, SqlReturn,
+    AttrCpMatch, AttrOdbcVersion, EnvironmentAttribute, FetchOrientation, HDbc, HEnv, Handle,
+    HandleType, SQLAllocHandle, SQLDataSourcesW, SQLDriversW, SQLSetEnvAttr,
 };
 use std::{convert::TryInto, ptr::null_mut};
 
@@ -59,45 +60,26 @@ impl Environment {
     /// > support connection pooling. This means the driver is able to handle a call on any thread
     /// > at any time and is able to connect on one thread, to use the connection on another thread,
     /// > and to disconnect on a third thread.
-    pub unsafe fn set_connection_pooling(
-        scheme: odbc_sys::AttrConnectionPooling,
-    ) -> Result<(), Error> {
-        match SQLSetEnvAttr(
+    pub unsafe fn set_connection_pooling(scheme: odbc_sys::AttrConnectionPooling) -> SqlResult<()> {
+        SQLSetEnvAttr(
             null_mut(),
             odbc_sys::EnvironmentAttribute::ConnectionPooling,
             scheme.into(),
             0,
-        ) {
-            SqlReturn::ERROR => Err(Error::NoDiagnostics),
-            SqlReturn::SUCCESS | SqlReturn::SUCCESS_WITH_INFO => Ok(()),
-            other => panic!(
-                "Unexpected Return value ('{:?}') for SQLSetEnvAttr then trying to set connection \
-                pooling to {:?}",
-                other, scheme
-            ),
-        }
+        )
+        .into_sql_result("SQLSetEnvAttr")
     }
 
-    pub fn set_connection_pooling_matching(
-        &mut self,
-        matching: odbc_sys::AttrCpMatch,
-    ) -> Result<(), Error> {
+    pub fn set_connection_pooling_matching(&mut self, matching: AttrCpMatch) -> SqlResult<()> {
         unsafe {
-            match SQLSetEnvAttr(
+            SQLSetEnvAttr(
                 self.handle,
                 odbc_sys::EnvironmentAttribute::CpMatch,
                 matching.into(),
                 0,
-            ) {
-                SqlReturn::ERROR => Err(Error::NoDiagnostics),
-                SqlReturn::SUCCESS | SqlReturn::SUCCESS_WITH_INFO => Ok(()),
-                other => panic!(
-                    "Unexpected Return value ('{:?}') for SQLSetEnvAttr then trying to set \
-                    connection pooling maching to {:?}",
-                    other, matching
-                ),
-            }
+            )
         }
+        .into_sql_result("SQLSetEnvAttr")
     }
 
     /// An allocated ODBC Environment handle
@@ -111,65 +93,36 @@ impl Environment {
     /// lead to race condition thus violating Rust's safety guarantees.
     ///
     /// Creating one environment in your binary is safe however.
-    pub unsafe fn new() -> Result<Self, Error> {
+    pub unsafe fn new() -> SqlResult<Self> {
         let mut handle = null_mut();
-        let (handle, info) = match SQLAllocHandle(HandleType::Env, null_mut(), &mut handle) {
-            // We can't provide nay diagnostics, as we don't have
-            SqlReturn::ERROR => return Err(Error::NoDiagnostics),
-            SqlReturn::SUCCESS => (handle, false),
-            SqlReturn::SUCCESS_WITH_INFO => (handle, true),
-            other => panic!(
-                "Unexpected Return value for allocating ODBC Environment: {:?}",
-                other
-            ),
-        };
-
-        debug!("ODBC Environment created.");
-
-        let env = Environment {
+        let result: SqlResult<()> = SQLAllocHandle(HandleType::Env, null_mut(), &mut handle)
+            .into_sql_result("SQLAllocHandle");
+        result.on_success(|| Environment {
             handle: handle as HEnv,
-        };
-        if info {
-            log_diagnostics(&env);
-        }
-        Ok(env)
+        })
     }
 
     /// Declares which Version of the ODBC API we want to use. This is the first thing that should
     /// be done with any ODBC environment.
-    pub fn declare_version(&self, version: AttrOdbcVersion) -> Result<(), Error> {
+    pub fn declare_version(&self, version: AttrOdbcVersion) -> SqlResult<()> {
         unsafe {
-            let result = SQLSetEnvAttr(
+            SQLSetEnvAttr(
                 self.handle,
                 EnvironmentAttribute::OdbcVersion,
                 version.into(),
                 0,
             )
-            .into_result(self, "SQLSetEnvAttr");
-
-            // Translate invalid attribute into a more meaningful error, provided the additional
-            // context that we know we tried to set version number.
-            result.map_err(|error| {
-                if let Error::Diagnostics { record, function } = error {
-                    if record.state == State::INVALID_STATE_TRANSACTION {
-                        Error::UnsupportedOdbcApiVersion(record)
-                    } else {
-                        Error::Diagnostics { record, function }
-                    }
-                } else {
-                    error
-                }
-            })
+            .into_sql_result("SQLSetEnvAttr")
         }
     }
 
     /// Allocate a new connection handle. The `Connection` must not outlive the `Environment`.
-    pub fn allocate_connection(&self) -> Result<Connection<'_>, Error> {
+    pub fn allocate_connection(&self) -> SqlResult<Connection<'_>> {
         let mut handle = null_mut();
         unsafe {
             SQLAllocHandle(HandleType::Dbc, self.as_handle(), &mut handle)
-                .into_result(self, "SQLAllocHandle")?;
-            Ok(Connection::new(handle as HDbc))
+                .into_sql_result("SQLAllocHandle")
+                .on_success(|| Connection::new(handle as HDbc))
         }
     }
 
@@ -178,7 +131,8 @@ impl Environment {
         self.handle
     }
 
-    /// List drivers descriptions and driver attribute keywords.
+    /// List drivers descriptions and driver attribute keywords. Returns `None` to indicate the end
+    /// of the list.
     ///
     /// # Safety
     ///
@@ -207,12 +161,12 @@ impl Environment {
         direction: FetchOrientation,
         buffer_description: &mut Vec<u16>,
         buffer_attributes: &mut Vec<u16>,
-    ) -> Result<bool, Error> {
+    ) -> Option<SqlResult<()>> {
         // Use full capacity
         buffer_description.resize(buffer_description.capacity(), 0);
         buffer_attributes.resize(buffer_attributes.capacity(), 0);
 
-        match SQLDriversW(
+        SQLDriversW(
             self.handle,
             direction,
             buffer_description.as_mut_ptr(),
@@ -221,17 +175,12 @@ impl Environment {
             buffer_attributes.as_mut_ptr(),
             buffer_attributes.len().try_into().unwrap(),
             null_mut(),
-        ) {
-            SqlReturn::NO_DATA => Ok(false),
-            other => {
-                other.into_result(self, "SQLDriversW")?;
-                Ok(true)
-            }
-        }
+        )
+        .into_opt_sql_result("SQLDriversW")
     }
 
-    /// Use together with [`Environment::drivers_buffer_fill`] to list drivers descriptions and driver attribute
-    /// keywords.
+    /// Use together with [`Environment::drivers_buffer_fill`] to list drivers descriptions and
+    /// driver attribute keywords.
     ///
     /// # Safety
     ///
@@ -257,12 +206,12 @@ impl Environment {
     pub unsafe fn drivers_buffer_len(
         &self,
         direction: FetchOrientation,
-    ) -> Result<Option<(i16, i16)>, Error> {
+    ) -> Option<SqlResult<(i16, i16)>> {
         // Lengths in characters minus terminating zero
         let mut length_description: i16 = 0;
         let mut length_attributes: i16 = 0;
         // Determine required buffer size
-        match SQLDriversW(
+        SQLDriversW(
             self.handle,
             direction,
             null_mut(),
@@ -271,11 +220,9 @@ impl Environment {
             null_mut(),
             0,
             &mut length_attributes,
-        ) {
-            SqlReturn::NO_DATA => return Ok(None),
-            other => other.into_result(self, "SQLDriversW")?,
-        }
-        Ok(Some((length_description, length_attributes)))
+        )
+        .into_opt_sql_result("SQLDriversW")
+        .map(|res| res.on_success(|| (length_description, length_attributes)))
     }
 
     /// Use together with [`Environment::data_source_buffer_fill`] to list drivers descriptions and
@@ -301,12 +248,12 @@ impl Environment {
     pub unsafe fn data_source_buffer_len(
         &self,
         direction: FetchOrientation,
-    ) -> Result<Option<(i16, i16)>, Error> {
+    ) -> Option<SqlResult<(i16, i16)>> {
         // Lengths in characters minus terminating zero
         let mut length_name: i16 = 0;
         let mut length_description: i16 = 0;
         // Determine required buffer size
-        match odbc_sys::SQLDataSourcesW(
+        odbc_sys::SQLDataSourcesW(
             self.handle,
             direction,
             null_mut(),
@@ -315,11 +262,9 @@ impl Environment {
             null_mut(),
             0,
             &mut length_description,
-        ) {
-            SqlReturn::NO_DATA => return Ok(None),
-            other => other.into_result(self, "SQLDataSourceW")?,
-        }
-        Ok(Some((length_name, length_description)))
+        )
+        .into_opt_sql_result("SQLDataSourceW")
+        .map(|res| res.on_success(|| (length_name, length_description)))
     }
 
     /// List drivers descriptions and driver attribute keywords.
@@ -348,12 +293,12 @@ impl Environment {
         direction: FetchOrientation,
         buffer_name: &mut Vec<u16>,
         buffer_description: &mut Vec<u16>,
-    ) -> Result<bool, Error> {
+    ) -> Option<SqlResult<()>> {
         // Use full capacity
         buffer_name.resize(buffer_name.capacity(), 0);
         buffer_description.resize(buffer_description.capacity(), 0);
 
-        match SQLDataSourcesW(
+        SQLDataSourcesW(
             self.handle,
             direction,
             buffer_name.as_mut_ptr(),
@@ -362,12 +307,7 @@ impl Environment {
             buffer_description.as_mut_ptr(),
             buffer_description.len().try_into().unwrap(),
             null_mut(),
-        ) {
-            SqlReturn::NO_DATA => Ok(false),
-            other => {
-                other.into_result(self, "SQLDataSourceW")?;
-                Ok(true)
-            }
-        }
+        )
+        .into_opt_sql_result("SQLDataSourceW")
     }
 }
