@@ -7,6 +7,7 @@
 mod connection;
 mod cursor;
 mod environment;
+mod error;
 mod execute;
 mod fixed_sized;
 mod into_parameter;
@@ -24,8 +25,9 @@ pub use self::{
     connection::{escape_attribute_value, Connection},
     cursor::{Cursor, CursorImpl, CursorRow, RowSetBuffer, RowSetCursor},
     environment::{DataSourceInfo, DriverCompleteOption, DriverInfo, Environment},
+    error::Error,
     fixed_sized::Bit,
-    handles::{ColumnDescription, DataType, Error, Nullability},
+    handles::{ColumnDescription, DataType, Nullability},
     into_parameter::IntoParameter,
     nullable::Nullable,
     parameter::{InputParameter, Out, Output, Parameter},
