@@ -171,7 +171,10 @@ where
     }
 
     unsafe fn bind_to_cursor(&mut self, cursor: &mut impl Cursor) -> Result<(), odbc_api::Error> {
-        cursor.stmt().bind_col(1, &mut self.column)?;
+        cursor
+            .stmt()
+            .bind_col(1, &mut self.column)
+            .into_result(cursor.stmt())?;
         Ok(())
     }
 }
